#####################################################################
# EJ(20170302)
# Auxiliary function
#####################################################################

getNoStks <- function(data, var, fun, var.name="N"){
	df0 <- aggregate(as.formula(paste(var, "~Year")), data, fun)
	df0$EcoRegion <- "ALL"
	df0 <- rbind(df0, aggregate(as.formula(paste(var, "~Year+EcoRegion")), data, fun))
	nms <- names(df0)
	nms[nms==var] <- var.name
	names(df0) <- nms
	df0
}

runDiagsME <- function(fit, me, data, file, nc, nd){
	require(lme4)
	require(influence.ME)
	require(lattice)
	pdf(file, paper="a4")
	# check homogeneity of variance
	# overall
	print(xyplot(residuals(fit)~predict(fit), main="homogeneity of variance"))
	# conditional 
	print(xyplot(residuals(fit)~predict(fit)|data[,me], main="homogeneity of variance", scales=list(x=list(relation="free"))))
	# check normality
	pfun <- function(x, ...){
		panel.qqmathline(x, col="gray50")
		panel.qqmath(x, ...)
	}
	# overall
	print(qqmath(residuals(fit), panel=pfun, main="normality of residuals", pch=19, cex=0.5))
	# conditional
	print(qqmath(~residuals(fit)|data[,me], panel=pfun, main="normality of residuals", pch=19, cex=0.5))
	# assessing the random effects
	print(dotplot(ranef(fit, condVar = TRUE), sub="Ramdom effects", main=FALSE))
	# influence
	ifl <- influence(fit, me)
	# dfbetas
	print(plot(ifl, "dfbetas", main="Influence measures - dfbetas"))
	# Cook's distance
	print(plot(ifl, "cook", main="Influence measures - cook's distance"))
	# stks influence in fixed effects
	ifl.stk <- influence.stk(fit, data, me, nc, nd)
	print(dotplot(eval(parse(text=paste(me, "~sd"))), data=ifl.stk, main="Influence in fixed effect \"year\""))
	dev.off()

}

influence.stk <- function(fit, data, me, nc, nd){
	require(parallel)
	stks <- unique(data[,me])
	stks <- split(stks, stks)
	lst <- mclapply(stks, function(x){
		refit <- update(fit, data=subset(data, eval(parse(text=paste(me, "!=x")))))
		sd(predict(refit, re.form=~0, type="response", newdata=nd)/predict(fit, re.form=~0, type="response", newdata=nd))
	}, mc.cores=nc)
	df0 <- data.frame(names(lst), unlist(lst))
	names(df0) <- c(me, "sd")
	df0
}





